#-*- coding:utf-8 -*-

"""
No rights reserved. All files in this repository are released into the public
domain.
"""
from libopensesame.py3compat import *
from libopensesame.item import item
from openexp.keyboard import keyboard
from libopensesame.exceptions import osexception
from libqtopensesame.items.qtautoplugin import qtautoplugin
from openexp.canvas import Canvas
from openexp.canvas_elements import *
import math
import pygame
import random
import numpy as np
import scipy.signal as signal
from pylsl import StreamInlet, resolve_stream
import csv

class Wailord(item):

	"""
	This class (the class with the same name as the module) handles the basicq
	functionality of the item. It does not deal with GUI stuff.
	"""

	# Provide an informative description for your plug-in.
	description = u'A full, customizeable rowing race component'

	def reset(self):
		self.var.boat_scale = 1
		self.var.OutDataFileName = u'c:\\rowfilecreated.csv'
		self.var.DataLen = 1000
 		self.var.DataDur = 240
 		self.var.ViewPortPerc = 100
 		self.var.save_dat = False
 		self.var.debug = False
 		self.var.Speed_Norm = 1
   		self.var.inputmode1 = u'File Input'
   		self.var.inputmode2 = u'File Input'
		self.var.inputparams1 = u'c:\\row1.csv'
		self.var.inputparams2 = u'c:\\row1.csv'
		self.var.EndTime = -1
		
	def prepare(self):
		if (self.var.inputmode1 != u'Continue') and (self.var.inputmode2 != u'Continue'):
			self.experiment.x = [0,0]
			self.experiment.boat_scale = self.var.boat_scale/100.0
			self.experiment.lsl_active = u'no'
			self.var.dat_active = u'no' 
			self.experiment.inputmode1 = self.var.inputmode1
			self.experiment.inputmode2 = self.var.inputmode2
			self.experiment.save_dat = self.var.save_dat
			
			if (self.experiment.save_dat == 'yes'):
				self.experiment.CSV_OutFile = open(self.var.OutDataFileName,'wb')
				self.experiment.CSVWriter = csv.writer(self.experiment.CSV_OutFile)
		
			# One inlet, even if we need two streams.
			if (self.var.inputmode1 == u'lsl Stream Input') or (self.var.inputmode2 == u'lsl Stream Input'):
				parms1 = self.var.inputparams1.split(',') 
				parms2 = self.var.inputparams2.split(',') 
				self.var.stream1index = int(parms1[0])
				self.var.stream2index = int(parms2[0])
				self.experiment.lsl_active = u'yes'
				self.dprint("Looking for a receivable EEG stream...")
				self.experiment.streams = resolve_stream('type', 'EEG')
				self.dprint("Opening the first EEG stream...")
				self.experiment.inlet = StreamInlet(self.experiment.streams[0])
			
			if (self.var.inputmode1 == u'File Input') or (self.var.inputmode2 == u'File Input'):
				self.experiment.dat_active = u'yes'
				if (self.var.inputmode1 == u'File Input'):
					parms = self.var.inputparams1.split(',') 
					self.dprint('Opening top rower data File: ' + parms[0] + ' channel ' + parms[1])
					self.experiment.dataFile1 = DataFile(parms[0], self.var, int(parms[1]))
				if (self.inputmode2 == u'File Input'):
					parms = self.var.inputparams2.split(',') 
					self.dprint('Opening bottom rower data File: ' + parms[0] + ' channel ' + parms[1])
					self.experiment.dataFile2 = DataFile(parms[0], self.var, int(parms[1]))
			
			self.experiment.rowingScene = Scene(self)
	
	def waitforStart(self):
		if (self.experiment.lsl_active == 'yes'):
			while True:
				chunk, timestamps = self.experiment.inlet.pull_chunk()
				if timestamps:
					if (chunk[0][0] > 1.4) or (chunk[0][1] > 1.4):
						break
	
	def run(self):
		self.waitforStart()
		self.var.startTime = self.clock.time()
		direction =[1,1]
		forcenow = [0,0]
		chunk = [0,0]
		while True:
			# enable escape
			self.experiment.rowingScene.Keyboard.flush()
			# how long am I rowing?
			timePassed = (self.clock.time() - self.var.startTime)
			# if we have an LSL stream:
			# get the force at this moment and use this as a value for 'displacement' or speed.
			# cumulative displacement is place.
			# we try to get the data as asked in the setup.... forcenow[2] is being asked for.
			
			if (self.experiment.lsl_active == 'yes'):
				chunk, timestamps = self.experiment.inlet.pull_chunk()
				if timestamps:
					if (self.experiment.inputmode1 == u'lsl Stream Input'):
						try:
							forcenow[0]     =  chunk[0][0]-1.2  # 1.2 is a 'zero value' for the lsl force stream. (parameter?)
						except IndexError:
							self.dprint ('Simple DataWarning for first rower')
							pass
							
					if (self.experiment.inputmode2 == u'lsl Stream Input'):
						try:
							forcenow[1]     =  chunk[0][1]-1.2  
						except IndexError:
							self.dprint ('Simple DataWarning for second rower')
							pass

			if (self.experiment.dat_active=='yes'):
				if (self.experiment.inputmode1 == u'File Input'):
					dum1 = self.experiment.dataFile1.getForceFromFile(timePassed)
					try:
						forcenow[0] = dum1[-1]
					except IndexError:
						self.dprint ('Simple DataWarning for first rower')
						pass
				if (self.experiment.inputmode2 == u'File Input'):
					dum2 = self.experiment.dataFile2.getForceFromFile(timePassed)
					try:
						forcenow[1] = dum2[-1]
					except IndexError:
						self.dprint ('Simple DataWarning for second rower')
						pass
			
			# Change these params (1.19994113 and 1.30702710) later. 
			# They are the means in the testfile here
			direction[0] = 2 if forcenow[0] > 1.19994113 else 1
			direction[1] = 2 if forcenow[1] > 1.30702710 else 1
			# Normalize the forcevalue and apply to the position vector.
			
			self.experiment.x[0] = self.experiment.x[0] + (self.var.Speed_Norm * forcenow[0])
			self.experiment.x[1] = self.experiment.x[1] + (self.var.Speed_Norm * forcenow[1])
			
			if (self.experiment.save_dat == 'yes'):
				self.experiment.CSVWriter.writerow([timePassed, forcenow[0], forcenow[1]])
				
			#draw the scene with the new positional parameters.
			self.experiment.rowingScene.Draw(self.experiment.x, direction, self.var)

			# END OF THE RACE: FINISH
			if max(self.experiment.x) > self.experiment.rowingScene.finishpos-self.experiment.rowingScene.Boat1.width:
				break

			# END OF THE RACE: TIME
			if self.var.EndTime > 0:
				if timePassed > self.var.EndTime:
					break
			
		# we get here at the finish.....
		startTime = self.clock.time()
		# wait a second before truly ending the race
		while True:
			self.experiment.rowingScene.Keyboard.flush()
			timePassed = (self.clock.time() - startTime)
			if timePassed > 1000:
				break
	
	def dprint(self, strng):
		if (self.var.debug == 'yes'):
			print 'dbg: (wailord) : ' + strng
		

class qtWailord(Wailord, qtautoplugin):

	"""
	This class handles the GUI aspect of the plug-in. By using qtautoplugin, we
	usually need to do hardly anything, because the GUI is defined in info.json.
	"""

	def __init__(self, name, experiment, script=None):

		"""
		Constructor.

		Arguments:
		name		--	The name of the plug-in.
		experiment	--	The experiment object.

		Keyword arguments:
		script		--	A definition script. (default=None)
		"""

		# We don't need to do anything here, except call the parent
		# constructors.
		Wailord.__init__(self, name, experiment, script)
		qtautoplugin.__init__(self, __file__)

	def init_edit_widget(self):

		"""
		Constructs the GUI controls. Usually, you can omit this function
		altogether, but if you want to implement more advanced functionality,
		such as controls that are grayed out under certain conditions, you need
		to implement this here.
		"""

		# First, call the parent constructor, which constructs the GUI controls
		# based on info.json.
		qtautoplugin.init_edit_widget(self)
		# If you specify a 'name' for a control in info.json, this control will
		# be available self.[name]. The type of the object depends on the
		# control. A checkbox will be a QCheckBox, a line_edit will be a
		# QLineEdit. Here we connect the stateChanged signal of the QCheckBox,
		# to the setEnabled() slot of the QLineEdit. This has the effect of
		# disabling the QLineEdit when the QCheckBox is uncheckhed. We also
		# explictly set the starting state.

class DataFile:
	__float__ = 1
	def __init__(self, filename, vars, other):
	
		self.filename = filename
		self.data=np.genfromtxt(filename,delimiter=',').astype("float")
		self.data = self.data[~np.isnan(self.data).any(axis=1)]
		self.lasttime = 0;
		self.meandat = [0,0,0]
		self.sumdat = [0,0,0]
		self.SR = 200.0

		self.Filter(50.0)

		#self.Other = 1 if np.std(self.data[:,1]) > np.std(self.data[:,2]) else 2
		self.Other = other
		# start at first stroke
		fsi = 	np.argmax(self.data[:,self.Other]>(2.2*np.std(self.data[:,self.Other]))+(np.mean(self.data[:,self.Other]))) 

		self.data = self.data[fsi:-1,:]
		self.data[:,0] = self.data[:,0]-self.data[0,0]
		
		# end at set time...
		eri =  np.argmax(self.data[:,0]>(vars.DataDur*1000))
		self.data = self.data[0:eri,:]
		self.cumsum = np.cumsum(self.data[:,self.Other])
      
	def Filter(self, cutoffFreq):
		# Filter the input so the 50 hz component is out.
		N  = 12    # Filter order
		Wn = 1.0/ (self.SR/cutoffFreq) # Cutoff frequency 
		B, A = signal.butter(N, Wn, output='ba')

		# time starts at 0 in this file...
		self.data[:,0] = (self.data[:,0]-self.data[0,0])*1000.0
	
		for i in range(1,3):
			self.data[:,i] = signal.filtfilt(B,A, self.data[:,i])

	def getForceFromFile(self, busyinms):
		datatimeindex1 = np.argmax(self.data[:,0] > (self.lasttime) )
		datatimeindex2 = np.argmax(self.data[:,0] > (busyinms) )
		forceNow = self.data[datatimeindex1:datatimeindex2, self.Other]
		self.lasttime = busyinms
		return forceNow
		
class Scene:
	__float__ = 2
	def __init__(self, parent):
		self.scale = 10*parent.experiment.boat_scale

		self.totallength = parent.var.width * (100.0 / parent.var.ViewPortPerc)
		self.finishpos = self.totallength - 50
		self.canvas = Canvas(parent.var.experiment,background_color = u'rgb(240,240,245)')

		self.totallength = self.canvas.width * (100.0 / parent.var.ViewPortPerc)
		self.stopscroll = self.totallength - self.canvas.width
		self.finishpos = self.totallength - 200
		
		self.positions = [-self.canvas.height/4, self.canvas.height/4]
		self.Keyboard = keyboard(parent.var.experiment)
		self.parent = parent.var
        
		self.Boat1 = Boat(parent.var, self.canvas, self.positions[0], 'Left',  self.scale)
		self.Boat2 = Boat(parent.var, self.canvas, self.positions[1], 'Right', self.scale)
		
		for f in [-1,0,1,2,3,4,5]:
			self.canvas['finish'+ str(f)] = Image(parent.experiment.pool[u'finish.png'],x=self.finishpos, y=-200+(f*100), scale = .15)
	
	def Draw(self, ypos, direction, var):
	
		start = min(ypos)
		start = min(start, self.stopscroll)
		
		for f in [-1,0,1,2,3,4,5]:
			self.canvas['finish'+ str(f)].x=(self.finishpos - start) - (var.width / 2)
            
		self.Boat1.Draw(ypos[0]-start, direction[0])
		self.Boat2.Draw(ypos[1]-start, direction[1])
		self.canvas.show()
		
class Boat:
	def __init__(self, parent, dCanvas, position, name, scale):
		self.position = position
		self.name = name
		self.scale = scale
		self.canvas = dCanvas
		self.parent = parent
		self.canvas[name + 'boat'] = Image(parent.experiment.pool[u'rowing1.png'],x=0, y=position, scale = self.scale)
		self.width = self.canvas[self.name +'boat'].width
	
	def Draw(self, newpos, direction):
		self.canvas[self.name +'boat'].fname = self.parent.experiment.pool[u'rowing'+str(direction)+'.png']
 		self.canvas[self.name +'boat'].x = (self.canvas[self.name +'boat'].width/2) + (newpos-(self.canvas.width/2))

